package com.baidu.flutter_bmfutils.Handlers;

import android.util.Log;

import com.baidu.flutter_bmfutils.Constants;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import main.com.baidu.flutter_bmfutils.Env;
import com.baidu.flutter_bmfutils.Handlers.MethodID.OpenMethodID;

import main.com.baidu.flutter_bmfutils.Handlers.CalculateHandler;
import main.com.baidu.flutter_bmfutils.Handlers.OpenPanoHandler;
import main.com.baidu.flutter_bmfutils.Handlers.VersionHandler;

import com.baidu.flutter_bmfutils.OpenBaiduMapPoiDetailHandler;
import com.baidu.flutter_bmfutils.Handlers.MethodID.CalcMethodID;
import com.baidu.flutter_bmfutils.Handlers.MethodID.CommonMethodID;
import com.baidu.flutter_bmfutils.Handlers.MethodChannelHandler;
import com.baidu.flutter_bmfutils.Handlers.OpenBaiduMapNaviHandler;
import com.baidu.flutter_bmfutils.Handlers.OpenBaiduRouteHandler;
import com.baidu.flutter_bmfutils.Handlers.OpenBaiduMapPoiNearHandler;

/**
 * method handle 工厂类
 */
public class HandlersFactory {

    public static final String TAG = com.baidu.flutter_bmfutils.Handlers.HandlersFactory.class.getSimpleName();
    private static volatile com.baidu.flutter_bmfutils.Handlers.HandlersFactory sInstance;

    public Map<Integer, MethodChannelHandler> methodHandlerMap = new HashMap<>();

     public static com.baidu.flutter_bmfutils.Handlers.HandlersFactory getInstance() {

            if (null == sInstance) {
                synchronized (com.baidu.flutter_bmfutils.Handlers.HandlersFactory.class) {
                    if (null == sInstance) {
                        sInstance = new com.baidu.flutter_bmfutils.Handlers.HandlersFactory();
                    }
                }
            }

            return sInstance;
       }

       private HandlersFactory(){
           methodHandlerMap.put(Constants.HandlerType.OPEN_NAVI_TYPE, new OpenBaiduMapNaviHandler());
           methodHandlerMap.put(Constants.HandlerType.OPEN_ROUTE_TYPE, new OpenBaiduRouteHandler());
           methodHandlerMap.put(Constants.HandlerType.OPEN_PANO_TYPE, new OpenPanoHandler());
           methodHandlerMap.put(Constants.HandlerType.OPEN_POI_DETAIL_TYPE, new OpenBaiduMapPoiDetailHandler());
           methodHandlerMap.put(Constants.HandlerType.OPEN_POI_NEAR_TYPE, new OpenBaiduMapPoiNearHandler());
           methodHandlerMap.put(Constants.HandlerType.CALCULATE_TYPE, new CalculateHandler());
           methodHandlerMap.put(Constants.HandlerType.GET_VERSUON_TYPE, new VersionHandler());
       }

       public void dispatchMethodHandler(MethodCall call, MethodChannel.Result result) {
           if(Env.DEBUG) {
               Log.d(TAG, "dispatchMethodHandler enter");
           }
          if(null == call || null == result){
              return;
          }

          MethodChannelHandler methodChannelHandler = null;
          String method = call.method;
          switch (method){
              case OpenMethodID.kOpenBaiduMapNavi:
                  methodChannelHandler = methodHandlerMap.get(Constants.HandlerType.OPEN_NAVI_TYPE);
                  break;
              case OpenMethodID.kOpenBaiduMapRoute:
                  methodChannelHandler = methodHandlerMap.get(Constants.HandlerType.OPEN_ROUTE_TYPE);
                  break;
              case OpenMethodID.kOpenBaiduMapPanorama:
                  methodChannelHandler = methodHandlerMap.get(Constants.HandlerType.OPEN_PANO_TYPE);
                  break;
              case OpenMethodID.kOpenBaiduMapPoiDetail:
                  methodChannelHandler = methodHandlerMap.get(Constants.HandlerType.OPEN_POI_DETAIL_TYPE);
                  break;
              case  OpenMethodID.kOpenBaiduMapPoiNear:
                  methodChannelHandler = methodHandlerMap.get(Constants.HandlerType.OPEN_POI_NEAR_TYPE);
                  break;
              case CalcMethodID.kCalculateRectArea:
              case CalcMethodID.kCalculatePolygonArea:
              case CalcMethodID.kCoordTypeConvert:
                  case CalcMethodID.kPolygonContainsCoord:
              case CalcMethodID.kCircleContainsCoord:
              case CalcMethodID.kLocationDistance:
                  case CalcMethodID.kNearestPointToLine:
                  methodChannelHandler = methodHandlerMap.get(Constants.HandlerType.CALCULATE_TYPE);
                  break;
              case CommonMethodID.kGetVersion:
                  methodChannelHandler = methodHandlerMap.get(Constants.HandlerType.GET_VERSUON_TYPE);
                  break;
              default:
                  break;
          }

          if(null != methodChannelHandler){
              methodChannelHandler.handlerMethodCallResult(call, result);
          }
       }
}