package com.baidu.flutter_bmfutils.Handlers;

import android.text.TextUtils;
import android.util.Log;

import com.baidu.flutter_bmfbase.BmfContextHelper;
import com.baidu.flutter_bmfutils.Constants;
import com.baidu.flutter_bmfutils.Constants.NaviType;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import main.com.baidu.flutter_bmfutils.Bean.OpenNaviOptionBean;
import main.com.baidu.flutter_bmfutils.Env;
import java.util.HashMap;

import com.baidu.mapapi.navi.BaiduMapNavigation;
import com.baidu.mapapi.navi.NaviParaOption;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.baidu.flutter_bmfutils.Handlers.MethodChannelHandler;
import com.baidu.flutter_bmfutils.Constants.ErrorCode;

/**
 * 打开百度导航
 */
public class OpenBaiduMapNaviHandler extends MethodChannelHandler {

    public static final String TAG = OpenBaiduMapNaviHandler.class.getSimpleName();


    @Override
    public void handlerMethodCallResult(MethodCall call, MethodChannel.Result result) {
        super.handlerMethodCallResult(call, result);
        if(Env.DEBUG) {
            Log.d(TAG, "handlerMethodCallResult enter"+call.arguments.toString());
        }

        if(null == call.arguments){
            returnResult(ErrorCode.OPEN_OPTION_NULL);
            return;
        }

        HashMap<String, Object> argumentsMap = (HashMap<String, Object>)call.arguments;
        if(null == argumentsMap){
            returnResult(ErrorCode.OPEN_OPTION_NULL);
            return;
        }

        HashMap<String, Object> naviOptions = (HashMap<String, Object>)argumentsMap.get("naviOption");
        if(null == naviOptions){
            returnResult(ErrorCode.OPEN_OPTION_NULL);
            return;
        }

        Gson gson = new GsonBuilder().enableComplexMapKeySerialization().create();
        String json = gson.toJson(naviOptions);
        if(TextUtils.isEmpty(json)){
            returnResult(ErrorCode.OPTION_ERROR);
            return;
        }

        OpenNaviOptionBean openNaviOptionBean = gson.fromJson(json, OpenNaviOptionBean.class);
        openBaiduNaviImp(openNaviOptionBean, result);
    }

    private void openBaiduNaviImp( OpenNaviOptionBean openNaviOptionBean, MethodChannel.Result result){
        if(null == BmfContextHelper.sContext){
            returnResult(ErrorCode.OPEN_OTHER_ERROR);
            return;
        }

        if(null == openNaviOptionBean){
            returnResult(ErrorCode.OPTION_ERROR);
            return;
        }

        NaviParaOption naviParaOption = toNaviParaOption(openNaviOptionBean);
        if(null == naviParaOption){
            returnResult(ErrorCode.OPTION_ERROR);
            return;
        }

        BaiduMapNavigation.setSupportWebNavi(openNaviOptionBean.isSupportWeb);

        boolean ret = false;
        try {
            switch (openNaviOptionBean.naviType){
                case NaviType.sDriveNavi:
                    ret = BaiduMapNavigation.openBaiduMapNavi(naviParaOption, BmfContextHelper.sContext);
                    break;
                case NaviType.sARWalkNavi:
                    ret = BaiduMapNavigation.openBaiduMapWalkNaviAR(naviParaOption, BmfContextHelper.sContext);
                    break;
                case NaviType.sRideNavi:
                    ret = BaiduMapNavigation.openBaiduMapBikeNavi(naviParaOption, BmfContextHelper.sContext);
                    break;
                case NaviType.sWalkNavi:
                    ret = BaiduMapNavigation.openBaiduMapWalkNavi(naviParaOption, BmfContextHelper.sContext);
                    break;
                default:
                    break;
            }
        }catch (Exception e){
            Log.e(TAG, e.toString());
        }

        if(ret){
            returnResult(ErrorCode.OPEN_NO_ERROR);
        }else{
            returnResult(ErrorCode.OPEN_OTHER_ERROR);
        }

    }

    private NaviParaOption toNaviParaOption(OpenNaviOptionBean openNaviOptionBean){
        if(null == openNaviOptionBean){
            return null;
        }

        NaviParaOption naviParaOption = new NaviParaOption();
        naviParaOption.endName(openNaviOptionBean.endName);
        naviParaOption.startName(openNaviOptionBean.startName);
        naviParaOption.startPoint(openNaviOptionBean.startCoord);
        naviParaOption.endPoint(openNaviOptionBean.endCoord);
        return naviParaOption;
    }
}