package com.baidu.flutter_bmfutils.Handlers;

import android.text.TextUtils;
import android.util.Log;

import com.baidu.flutter_bmfbase.BmfContextHelper;
import com.baidu.flutter_bmfutils.Constants;
import com.baidu.flutter_bmfutils.Handlers.MethodChannelHandler;
import com.baidu.mapapi.navi.BaiduMapNavigation;
import com.baidu.mapapi.navi.NaviParaOption;
import com.baidu.mapapi.utils.route.BaiduMapRoutePlan;
import com.baidu.mapapi.utils.route.RouteParaOption;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.util.HashMap;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import main.com.baidu.flutter_bmfutils.Bean.OpenNaviOptionBean;
import main.com.baidu.flutter_bmfutils.Bean.OpenRouteOptionBean;
import main.com.baidu.flutter_bmfutils.Env;

/**
 * 打开路径规划
 */
public class OpenBaiduRouteHandler extends MethodChannelHandler {

    public static final String TAG = OpenBaiduRouteHandler.class.getSimpleName();

    @Override
    public void handlerMethodCallResult(MethodCall call, MethodChannel.Result result) {
        super.handlerMethodCallResult(call, result);
        if(Env.DEBUG) {
            Log.d(TAG, "handlerMethodCallResult enter"+call.arguments.toString());
        }

        if(null == call.arguments){
            returnResult(Constants.ErrorCode.OPEN_OPTION_NULL);
            return;
        }

        HashMap<String, Object> argumentsMap = (HashMap<String, Object>)call.arguments;
        if(null == argumentsMap){
            returnResult(Constants.ErrorCode.OPEN_OPTION_NULL);
            return;
        }

        HashMap<String, Object> naviOptions = (HashMap<String, Object>)argumentsMap.get("routeOption");
        if(null == naviOptions){
            returnResult(Constants.ErrorCode.OPTION_ERROR);
            return;
        }

        Gson gson = new GsonBuilder().enableComplexMapKeySerialization().create();
        String json = gson.toJson(naviOptions);
        if(TextUtils.isEmpty(json)){
            returnResult(Constants.ErrorCode.OPTION_ERROR);
            return;
        }

        OpenRouteOptionBean openRouteOptionBean = gson.fromJson(json, OpenRouteOptionBean.class);
        openBaiduNaviImp(openRouteOptionBean, result);
    }

    private void openBaiduNaviImp( OpenRouteOptionBean openRouteOptionBean, MethodChannel.Result result){
        if(null == BmfContextHelper.sContext){
            returnResult(Constants.ErrorCode.OPEN_OTHER_ERROR);
            return;
        }

        if(null == openRouteOptionBean){
            returnResult(Constants.ErrorCode.OPTION_ERROR);
            return;
        }

        RouteParaOption routeParaOption = toNaviParaOption(openRouteOptionBean);
        if(null == routeParaOption){
            returnResult(Constants.ErrorCode.OPTION_ERROR);
            return;
        }

        BaiduMapRoutePlan.setSupportWebRoute(openRouteOptionBean.isSupportWeb);
        boolean ret = false;
        try {
            switch (openRouteOptionBean.routeType){
                case Constants.RouteType.sWalkingRoute:
                    ret = BaiduMapRoutePlan.openBaiduMapWalkingRoute(routeParaOption, BmfContextHelper.sContext);
                    break;
                case Constants.RouteType.sDrivingRoute:
                    ret = BaiduMapRoutePlan.openBaiduMapDrivingRoute(routeParaOption, BmfContextHelper.sContext);
                    break;
                case Constants.RouteType.sTransitRoute:
                    ret = BaiduMapRoutePlan.openBaiduMapTransitRoute(routeParaOption, BmfContextHelper.sContext);
                    break;
                default:
                    break;
            }
        }catch (Exception e){
            Log.e(TAG, e.toString());
        }

        if(true == ret) {
            returnResult(Constants.ErrorCode.OPEN_NO_ERROR);
        }else{
            returnResult(Constants.ErrorCode.OPEN_OTHER_ERROR);
        }
    }

    private RouteParaOption toNaviParaOption(OpenRouteOptionBean openRouteOptionBean){
        if(null == openRouteOptionBean){
            return null;
        }


        RouteParaOption routeParaOption = new RouteParaOption();
        routeParaOption.busStrategyType(correctBusStrategy(openRouteOptionBean.transitPolicy));
        routeParaOption.startName(openRouteOptionBean.startName);
        routeParaOption.endName(openRouteOptionBean.endName);
        routeParaOption.startPoint(openRouteOptionBean.startCoord);
        routeParaOption.endPoint(openRouteOptionBean.endCoord);


        return routeParaOption;
    }

    /**
     * flutter传过来的BusStrategy 与android端不一致，需要进行修正
     */
    private RouteParaOption.EBusStrategyType  correctBusStrategy(int transitPolicy){
        transitPolicy -= 3;
        if(0 == transitPolicy){
            transitPolicy = 4;
        }else if(4 == transitPolicy){
            transitPolicy = 0;
        }

        return RouteParaOption.EBusStrategyType.values()[transitPolicy];
    }
}