package main.com.baidu.flutter_bmfutils.Handlers;

import android.text.TextUtils;
import android.util.Log;

import com.baidu.flutter_bmfbase.BmfContextHelper;
import com.baidu.flutter_bmfutils.Constants;
import com.baidu.flutter_bmfutils.Handlers.MethodChannelHandler;
import com.baidu.mapapi.utils.OpenClientUtil;
import com.baidu.mapapi.utils.poi.BaiduMapPoiSearch;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.util.HashMap;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import main.com.baidu.flutter_bmfutils.Bean.PanoOptionBean;
import main.com.baidu.flutter_bmfutils.Env;

/**
 * 打开全景
 */
public class OpenPanoHandler extends MethodChannelHandler {

    public  static final String TAG = OpenPanoHandler.class.getSimpleName();

    @Override
    public void handlerMethodCallResult(MethodCall call, MethodChannel.Result result) {
        super.handlerMethodCallResult(call,result);
        if(Env.DEBUG) {
            Log.d(TAG, "handlerMethodCallResult enter"+call.arguments.toString());
        }

        if(null == call.arguments){
            returnResult(Constants.ErrorCode.OPEN_OPTION_NULL);
            return;
        }

        HashMap<String, Object> argumentsMap = (HashMap<String, Object>)call.arguments;
        if(null == argumentsMap){
            returnResult(Constants.ErrorCode.OPEN_OPTION_NULL);
            return;
        }

        HashMap<String, Object> naviOptions = (HashMap<String, Object>)argumentsMap.get("panoramaOption");
        if(null == naviOptions){
            returnResult(Constants.ErrorCode.OPEN_OPTION_NULL);
            return;
        }

        Gson gson = new GsonBuilder().enableComplexMapKeySerialization().create();
        String json = gson.toJson(naviOptions);
        if(TextUtils.isEmpty(json)){
            returnResult(Constants.ErrorCode.OPTION_ERROR);
            return;
        }

        PanoOptionBean panoOptionBean = gson.fromJson(json, PanoOptionBean.class);
        openBaiduPanoImp(panoOptionBean, result);

    }

    private void openBaiduPanoImp(PanoOptionBean panoOptionBean, MethodChannel.Result result){
        if(null == BmfContextHelper.sContext){
            returnResult(Constants.ErrorCode.OPEN_OTHER_ERROR);
            return;
        }


        if(0 == OpenClientUtil.getBaiduMapVersion(BmfContextHelper.sContext)){
            returnResult(Constants.ErrorCode.OPEN_NOT_SUPPORT);
            return;
        }

        try {
            BaiduMapPoiSearch.openBaiduMapPanoShow(panoOptionBean.poiUid, BmfContextHelper.sContext);
        }catch (Exception e){
            returnResult(Constants.ErrorCode.OPEN_OTHER_ERROR);
            Log.e(TAG, e.toString());
        }
    }
}
