library bmfmap_utils.model.bmf_open_poi_options;

import 'package:flutter/material.dart';
import 'package:flutter_bmfbase/BaiduMap/bmfmap_base.dart'
    show BMFModel, BMFCoordinate;

/// 调起百度地图： poi详情参数类
class BMFOpenPoiDetailOption implements BMFModel {
  static BMFOpenPoiDetailOption poiDetailOption() =>
      BMFOpenPoiDetailOption(poiUid: null);

  /// poi的uid
  String poiUid;

  /// 指定返回自定义scheme
  ///
  /// 仅iOS支持，Android不支持
  String appScheme;

  /// 调起百度地图客户端失败后，是否支持调起web地图，默认：true
  bool isSupportWeb;

  /// BMFOpenPoiDetailOption构造方法
  BMFOpenPoiDetailOption(
      {@required this.poiUid, this.appScheme, this.isSupportWeb = true});

  @override
  fromMap(Map map) {
    return new BMFOpenPoiDetailOption(
        poiUid: map['poiUid'],
        appScheme: map['appScheme'],
        isSupportWeb: map['isSupportWeb'] as bool);
  }

  @override
  Map<String, Object> toMap() {
    return {
      'poiUid': this.poiUid,
      'appScheme': this.appScheme,
      'isSupportWeb': this.isSupportWeb
    };
  }
}

/// 调起百度地图： poi周边参数类
class BMFOpenPoiNearOption implements BMFModel {
  static BMFOpenPoiNearOption nearbyOption() =>
      BMFOpenPoiNearOption(location: null);

  /// 中心点（经纬度）
  BMFCoordinate location;

  /// 半径
  int radius;

  /// 关键词
  String keyword;

  /// 指定返回自定义scheme  (ios)
  String appScheme;

  /// 调起百度地图客户端失败后，是否支持调起web地图，默认：true  (ios)
  bool isSupportWeb;

  /// BMFOpenPoiNearOption 构造方法
  BMFOpenPoiNearOption(
      {@required this.location,
      this.radius,
      this.keyword,
      this.appScheme,
      this.isSupportWeb = true});

  @override
  fromMap(Map map) {
    return new BMFOpenPoiNearOption(
        location: BMFCoordinate.coordinate().fromMap(map['location']),
        radius: map['radius'] as int,
        keyword: map['keyword'],
        appScheme: map['appScheme'],
        isSupportWeb: map['isSupportWeb'] as bool);
  }

  @override
  Map<String, Object> toMap() {
    return {
      'location': this.location?.toMap(),
      'radius': this.radius,
      'keyword': this.keyword,
      'appScheme': this.appScheme,
      'isSupportWeb': this.isSupportWeb
    };
  }
}
