library bmfmap_utils.model.bmf_open_route_options;

import 'package:flutter/material.dart';
import 'package:flutter_bmfbase/BaiduMap/bmfmap_base.dart'
    show BMFModel, BMFCoordinate;

/// 调起客户端的导航类型
enum BMFOpenRouteType {
  ///<调起百度地图步行路线
  WalkingRoute,

  ///<调起百度地图驾车路线
  DrivingRoute,

  ///<调起百度地图公共交通路线
  TransitRoute,
}

/// 交通路线策略
enum BMFOpenTransitPolicy {
  ///<推荐
  RECOMMAND,

  ///<少换乘
  TRANSFER_FIRST,

  ///<少步行
  WALK_FIRST,

  ///<不坐地铁
  NO_SUBWAY,

  ///<时间短
  TIME_FIRST,
}

/// 调起客户端路线参数
class BMFOpenRouteOption implements BMFModel {
  static BMFOpenRouteOption routeOption() =>
      BMFOpenRouteOption(startCoord: null, endCoord: null, routeType: null);

  /// 路线起点， 经纬度坐标
  BMFCoordinate startCoord;

  /// 路线起点名称
  String startName;

  /// 路线终点， 经纬度坐标
  BMFCoordinate endCoord;

  /// 路线终点名称
  String endName;

  /// 调起路线类型
  BMFOpenRouteType routeType;

  /// 交通路线策略 默认T: RANSIT_RECOMMAND
  /// 异常值时: 强制使用RECOMMAND
  /// 调起百度地图公共交通路线时必填参数
  BMFOpenTransitPolicy transitPolicy;

  /// 指定返回自定义scheme (ios)
  String appScheme;

  /// 调起百度地图客户端失败后，是否支持调起web地图，默认：true
  bool isSupportWeb;

  /// BMFOpenRouteOption 构造方法
  BMFOpenRouteOption(
      {@required this.startCoord,
      this.startName,
      @required this.endCoord,
      this.endName,
      @required this.routeType,
      this.transitPolicy = BMFOpenTransitPolicy.RECOMMAND,
      this.appScheme,
      this.isSupportWeb = true});

  @override
  fromMap(Map map) {
    return new BMFOpenRouteOption(
        startCoord: BMFCoordinate.coordinate().fromMap(map['startCoord']),
        endCoord: BMFCoordinate.coordinate().fromMap(map['endCoord']),
        startName: map['startName'],
        endName: map['endName'],
        routeType: BMFOpenRouteType.values[map['naviType'] as int],
        transitPolicy:
            BMFOpenTransitPolicy.values[(map['transitPolicy'] as int) - 3],
        appScheme: map['appScheme'],
        isSupportWeb: map['isSupportWeb'] as bool);
  }

  @override
  Map<String, Object> toMap() {
    return {
      'startCoord': this.startCoord?.toMap(),
      'startName': this.startName,
      'endCoord': this.endCoord?.toMap(),
      'endName': this.endName,
      'routeType': this.routeType.index,
      'transitPolicy': this.transitPolicy.index + 3, // native 初始值为3
      'appScheme': this.appScheme,
      'isSupportWeb': this.isSupportWeb
    };
  }
}
